/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: config_spam_rule.c,v 1.26 2003/02/14 00:23:35 burton Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

/*
  Routines to load global and user spam rule configs
*/
#include <string.h>

#include <liblm.h>

#include "config_msd.h"
#include "config_common.h"
#include "config_spam_rule.h"
#include "config_misc_rule.h"
#include "db.h"
#include "strlist.h"

config_spam_rule_t global_spam_rules;

int config_spam_rule_init_defaults(config_spam_rule_t *rules);
int config_spam_rule_init_defaults_from_globals(config_spam_rule_t* rules, config_spam_rule_t* global_rules);
int config_spam_rule_load_lists(config_spam_rule_t *rules, config_spam_rule_t *global_rules);
int config_spam_rule_load_variables(config_spam_rule_t *rules);

int config_spam_rule_verify(config_spam_rule_t *rules)
{
  return 0;
}

#ifdef USE_LM_DATABASE
int config_spam_rule_load_db(lm_string_t addr, 
			     config_spam_rule_t *rules)
{
  int retval;
  char buffer[1024];
  char *escuser = NULL;
  char *escdomain = NULL;
	
  msd_db_get_user_domain(&escuser, &escdomain, addr);
	
  /* Setup the database paramters for the user */
  snprintf(buffer, sizeof(buffer),
	   /*"and name='%s' and domain='%s'",*/
	   "and name=actual_name('%s','%s',4) and domain='%s'",
	   escuser, escdomain, escdomain);
  LM_STRING_NEW(dbwhere, buffer);
  retval = LM_CONFIG_DB_SET_PARAM(db, config.spamtable,
				  dbselectcol, dbkeycol, dbwhere);
  LM_STRING_FREE(dbwhere);
	
  /* Load the spam rules for the set config */
  if(retval == 0)
    {
      /* load the user rules and append global rules as per user configuration */
      retval = config_spam_rule_read(&global_spam_rules, rules);
    }
  /* Close the configuration that was opened */
  LM_CONFIG_CLOSE;

  LM_FREE(escuser);
  LM_FREE(escdomain);

  return retval;
}
#endif

int config_spam_rule_load_spamdir(config_spam_rule_t *rules, 
				  lm_string_t dotspam)
{
  int retval;
  
  /* Stat the dotspam file 
   * The check user functions return a trailing / on filenames if they 
   * happen to be a directory so we don't have to stat the file again
   */
  /* If the file is a directory */
  if(LM_STRING_BUFFER(dotspam)[LM_STRING_LEN(dotspam)-1] == '/')
    {
      /* load user rules as control files */
      SYSLOG((LOG_INFO, "Loading spam rule control files from %s", LM_STRING_BUFFER(dotspam)));
      retval = LM_CONFIG_CONTROL_SET_PARAM(dotspam);
    }
  else
    {
      SYSLOG((LOG_WARNING, "This version supports spam control directories only! Spam rules not loaded."));
      return -1;
    }
  
  /* Load the spam rules for the set config */
  if(retval == 0)
    {
      /* load the user rules and append global rules as per user configuration */
      retval = config_spam_rule_read(&global_spam_rules, rules);
    }
  /* Close the configuration that was opened */
  LM_CONFIG_CLOSE;
  
  return retval;
}

int config_spam_rule_load(lm_string_t addr, config_spam_rule_t *rules,
			  lm_string_t dotspam)
{
  int retval = 0;
  
  LM_ASSERT(rules != NULL);
  switch(config.compat_type)
    {
    case MSD_COMPAT_VPOP:
    case MSD_COMPAT_QMAIL:
      LM_ASSERT(dotspam != NULL);
      retval = config_spam_rule_load_spamdir(rules, dotspam);
      break;
#ifdef USE_LM_DATABASE      
    case MSD_COMPAT_MAGIC:
      LM_ASSERT(addr != NULL);
      if(db != NULL)
	retval = config_spam_rule_load_db(addr, rules);
      else
	retval = -1;
      break;
#endif
    default:
      SYSLOG((LOG_ERR, "Invalid compat type!"));
      retval = -1;
    }
  return retval;
}

/* Load the configuration files for spam checking, appending the global rule lists 
 * if desired */
int config_spam_rule_read(config_spam_rule_t* global_rules, config_spam_rule_t *rules)
{
  int retval = 0;

  LM_ASSERT(rules != NULL);
  /* Don't really need this since we know we will be filling in
     all of the variables in the structure */
  memset(rules, 0, sizeof(*rules));

  /* Load the special key to specify to use all global rules so
     we know how to initialize the structure */
  rules->all_global_rules = 0;
  if(global_rules != NULL)
    {
      LOAD_BOOL(SPAM, (NODEPS), ALL_GLOBAL_RULES, all_global_rules);
    }

  /* If the user specified to use all global rules, set the default values to those
   * in the globals, else set to the ones specified in the documentation
   * This only applies to boolean rules here.  The lists are also inherited later in the code.
   */
  if(rules->all_global_rules == 0)
    retval |= config_spam_rule_init_defaults(rules);
  else
    retval |= config_spam_rule_init_defaults_from_globals(rules, global_rules);

  /* Load the boolean variables */
  if(retval == 0)
    retval |= config_spam_rule_load_variables(rules);

  /* Load the white and black lists */
  if(retval == 0)
    retval |= config_spam_rule_load_lists(rules, global_rules);

  /* Verify that the configuration is ok */
  if(retval == 0)
    retval |= config_spam_rule_verify(rules);

  return retval;
}

/* 
 * All the code after this will be automatically generated in the 
 * future from spam_rules.in
 */

/* Free all the strings from the ruleset */
int config_spam_rule_free(config_spam_rule_t *rules)
{
  if(rules->quarantine_ext) LM_STRING_FREE(rules->quarantine_ext);
  if(rules->from_whitelist) STRLIST_FREE(rules->from_whitelist);
  if(rules->from_blacklist) STRLIST_FREE(rules->from_blacklist);
  if(rules->domain_whitelist) STRLIST_FREE(rules->domain_whitelist);
  if(rules->domain_blacklist) STRLIST_FREE(rules->domain_blacklist);
  if(rules->country_whitelist) STRLIST_FREE(rules->country_whitelist);
  if(rules->country_blacklist) STRLIST_FREE(rules->country_blacklist);
  if(rules->ip_whitelist) STRLIST_FREE(rules->ip_whitelist);
  if(rules->ip_blacklist) STRLIST_FREE(rules->ip_blacklist);
  
  return 0;
}

int config_spam_rule_init_defaults_from_globals(config_spam_rule_t* rules, config_spam_rule_t* global_rules)
{
  int retval = 0;

  rules->all_global_rules = global_rules->all_global_rules;
  rules->spam_check = global_rules->spam_check;
  rules->smtp_check = global_rules->smtp_check;
  rules->delivery_check = global_rules->delivery_check;
  rules->smtp_blocking = global_rules->smtp_blocking;
  if(rules->quarantine_ext) LM_STRING_FREE(rules->quarantine_ext);
  retval |= LM_STRING_DUP(rules->quarantine_ext,
			  global_rules->quarantine_ext);
  rules->require_full_addr = global_rules->require_full_addr;
  rules->block_mail_from_self = global_rules->block_mail_from_self;
  rules->block_ip_in_addr = global_rules->block_ip_in_addr;
  rules->valid_from_domain = global_rules->valid_from_domain;
  rules->valid_bounce = global_rules->valid_bounce;
  rules->require_helo = global_rules->require_helo;
  rules->check_ip_reverse_dns = global_rules->check_ip_reverse_dns;

  retval |= STRLIST_NEW(rules->from_blacklist, emptystring);
  retval |= STRLIST_NEW(rules->from_whitelist, emptystring);
  retval |= STRLIST_NEW(rules->domain_blacklist, emptystring);
  retval |= STRLIST_NEW(rules->domain_whitelist, emptystring);
  retval |= STRLIST_NEW(rules->country_blacklist, emptystring);
  retval |= STRLIST_NEW(rules->country_whitelist, emptystring);
  retval |= STRLIST_NEW(rules->ip_blacklist, emptystring);
  retval |= STRLIST_NEW(rules->ip_whitelist, emptystring);
  
  return retval;
}

int config_spam_rule_init_defaults(config_spam_rule_t* rules)
{
  int retval = 0;

  rules->all_global_rules = SPAM_CONTROL_ALL_GLOBAL_RULES_DEFAULT;
  rules->spam_check = SPAM_CONTROL_SPAM_CHECK_DEFAULT;
  rules->smtp_check = SPAM_CONTROL_SMTP_CHECK_DEFAULT;
  rules->delivery_check = SPAM_CONTROL_DELIVERY_CHECK_DEFAULT;
  rules->smtp_blocking = SPAM_CONTROL_SMTP_BLOCKING_DEFAULT;
  if(rules->quarantine_ext) LM_STRING_FREE(rules->quarantine_ext);
  retval |= LM_STRING_NEW(rules->quarantine_ext,
			  SPAM_CONTROL_QUARANTINE_EXT_DEFAULT);
  rules->require_full_addr = SPAM_CONTROL_REQUIRE_FULL_ADDR_DEFAULT;
  rules->block_mail_from_self = SPAM_CONTROL_BLOCK_MAIL_FROM_SELF_DEFAULT;
  rules->block_ip_in_addr = SPAM_CONTROL_BLOCK_IP_IN_ADDR_DEFAULT;
  rules->valid_from_domain = SPAM_CONTROL_VALID_FROM_DOMAIN_DEFAULT;
  rules->valid_bounce = SPAM_CONTROL_VALID_BOUNCE_DEFAULT;
  rules->require_helo = SPAM_CONTROL_REQUIRE_HELO_DEFAULT;
  rules->valid_helo_domain = SPAM_CONTROL_VALID_HELO_DOMAIN_DEFAULT;
  rules->check_ip_reverse_dns = SPAM_CONTROL_CHECK_IP_REVERSE_DNS_DEFAULT;
  
  retval |= STRLIST_NEW(rules->from_blacklist, emptystring);
  retval |= STRLIST_NEW(rules->from_whitelist, emptystring);
  retval |= STRLIST_NEW(rules->domain_blacklist, emptystring);
  retval |= STRLIST_NEW(rules->domain_whitelist, emptystring);
  retval |= STRLIST_NEW(rules->country_blacklist, emptystring);
  retval |= STRLIST_NEW(rules->country_whitelist, emptystring);
  retval |= STRLIST_NEW(rules->ip_blacklist, emptystring);
  retval |= STRLIST_NEW(rules->ip_whitelist, emptystring);
  
  return retval;
}		

int config_spam_rule_load_variables(config_spam_rule_t *rules)
{
  int retval = 0;

  LOAD_BOOL(SPAM, (NODEPS),
	    ALL_GLOBAL_RULES, all_global_rules);
  LOAD_BOOL(SPAM, (NODEPS),
	    SPAM_CHECK, spam_check);
  LOAD_BOOL(SPAM, (rules->spam_check),
	    SMTP_CHECK, smtp_check);
  LOAD_BOOL(SPAM, (rules->delivery_check),
	    DELIVERY_CHECK, delivery_check);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check),
	    SMTP_BLOCKING, smtp_blocking);
  LOAD_STRING(SPAM, (rules->spam_check && rules->smtp_check && !rules->smtp_blocking),
	      QUARANTINE_EXT, quarantine_ext);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check),
	    REQUIRE_FULL_ADDR, require_full_addr);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check),
	    BLOCK_MAIL_FROM_SELF, block_mail_from_self);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check),
	    BLOCK_IP_IN_ADDR, block_ip_in_addr);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check),
	    VALID_FROM_DOMAIN, valid_from_domain);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check),
	    VALID_BOUNCE, valid_bounce);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check),
	    REQUIRE_HELO, require_helo);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check && rules->require_helo),
	    VALID_HELO_DOMAIN, valid_helo_domain);
  LOAD_BOOL(SPAM, (rules->spam_check && rules->smtp_check),
	    CHECK_IP_REVERSE_DNS, check_ip_reverse_dns);
  
  return retval;
}

int config_spam_rule_load_lists(config_spam_rule_t *rules, config_spam_rule_t *global_rules)
{
  int retval = 0;

  LOAD_LIST(SPAM, (rules->spam_check && rules->smtp_check && rules->valid_from_domain),
	    FROM_BLACKLIST, from_blacklist);
  LOAD_LIST(SPAM, (rules->spam_check && rules->smtp_check && rules->valid_from_domain),
	    FROM_WHITELIST, from_whitelist);
  LOAD_LIST(SPAM, (rules->spam_check && rules->smtp_check),
	    DOMAIN_WHITELIST, domain_whitelist);
  LOAD_LIST(SPAM, (rules->spam_check && rules->smtp_check),
	    DOMAIN_BLACKLIST, domain_blacklist);
  LOAD_LIST(SPAM, (rules->spam_check && rules->smtp_check && rules->valid_from_domain),
	    COUNTRY_WHITELIST, country_whitelist);
  LOAD_LIST(SPAM, (rules->spam_check && rules->smtp_check && rules->valid_from_domain),
	    COUNTRY_BLACKLIST, country_blacklist);
  LOAD_LIST(SPAM, (rules->spam_check && rules->smtp_check),
	    IP_WHITELIST, ip_whitelist);
  LOAD_LIST(SPAM, (rules->spam_check && rules->smtp_check),
	    IP_BLACKLIST, ip_blacklist);

  return retval;
}
